//------------------------------------------------------------------------------
//
//  npglut.c
//  ANTZ application based on npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#include "npglut.h"


#include "../npio.h"

#include "npgl.h"
#include "npkey.h"


#define kWindowWidth 640
#define kWindowHeight 480
#define kWindowPositionX 50
#define kWindowPositionY 50


void npGlutInitGL();
void npGlutDrawGLScene(void);
void npGlutDrawGLSceneIdle(void);
void npGlutResizeGLScene(int Width, int Height);

void npGlutKeyDown (unsigned char key, int x, int y);
void npGlutKeyUp (unsigned char key, int x, int y);
void npGlutKeyDownSpecial (int key, int x, int y);
void npGlutKeyUpSpecial (int key, int x, int y);


//------------------------------------------------------------------------------
void npInitGlut (int argc, char **argv, void* dataRef)
{
	glutInit (&argc, argv);

	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);

//	glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);		//GLUT_RGB
	glutInitWindowPosition (kWindowPositionX, kWindowPositionY);
	glutInitWindowSize (kWindowWidth, kWindowHeight);
	glutCreateWindow ("ANTz");
	glutFullScreen ();

	npInitGL (dataRef);

	glutKeyboardFunc (npGlutKeyDown);
	glutKeyboardUpFunc (npGlutKeyUp);
	glutSpecialFunc (npGlutKeyDownSpecial);
	glutSpecialUpFunc (npGlutKeyUpSpecial);
	
	//Mouse Event Register with GLUT
	glutMouseFunc (npMouseEvent);
	glutMotionFunc (npMouseMotion);

	glutDisplayFunc (npGlutDrawGLScene);
	glutIdleFunc (npGlutDrawGLSceneIdle);

	glutReshapeFunc (npGLResizeScene);
}


//------------------------------------------------------------------------------
void npCloseGlut (void* dataRef)
{	return;
}


//------------------------------------------------------------------------------
void npAppLoopGlut (void* data)
{
	glutMainLoop();
}


//------------------------------------------------------------------------------
void npGlutDrawGLScene(void) 
{
	static int previousTime = 0;
	int time = 0;
	int deltaTime = 0;

	npGLDrawScene (npDataGetDataRef());

	glutSwapBuffers();
//	glutPostRedisplay();

	while (deltaTime < 12)
	{
	time = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = time - previousTime;
	}

	printf("\n %d   %d", deltaTime, time);

//	glfwSleep((double)0.1f);

	previousTime = time;
}

//------------------------------------------------------------------------------

void npGlutDrawGLSceneIdle(void)
{
	npGlutDrawGLScene();
	return;
}

//------------------------------------------------------------------------------
void npGlutFullScreenExit()
{
    glutPositionWindow (kWindowPositionX, kWindowPositionY);
	glutReshapeWindow (kWindowWidth, kWindowHeight);        /* Restore us */
	
	glutSwapBuffers();
//	glutPostRedisplay();
}


//------------------------------------------------------------------------------
void npGlutKeyDown(unsigned char key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyDown, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyUp(unsigned char key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyUp, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyDownSpecial(int key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyDownSpecial, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyUpSpecial(int key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyUpSpecial, glutGetModifiers() );
}


//------------------------------------------------------------------------------
void npGLSolidSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutSolidSphere (radius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutWireSphere (radius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLSolidCone (GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutSolidCone (base, height, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireCone (GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutWireCone (base, height, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLSolidTorus (GLdouble innerRadius, GLdouble outerRadius, 
					GLint slices, GLint stacks)
{
	glutSolidTorus (innerRadius, outerRadius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireTorus (GLdouble innerRadius, GLdouble outerRadius, 
					GLint slices, GLint stacks)
{
	glutWireTorus (innerRadius, outerRadius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLPrimitive (int primitive)
{
	switch (primitive)
	{
	case kNPglutWireCube : glutWireCube(2.0f); break;
	case kNPglutSolidCube : glutSolidCube(2.0f); break;

	case kNPglutWireSphere : glutWireSphere( 1.0f, 15, 15 ); break;
	case kNPglutSolidSphere : glutSolidSphere( 1.0f, 15, 15 ); break;

	case kNPglutWireCone : glutWireCone( 1.0f, 2.0f, 20, 1 ); break;
	case kNPglutSolidCone : glutSolidCone( 1.0f, 2.0f, 20, 1 ); break;

	case kNPglutWireTorus : glutWireTorus( 1.0f, 1.2f, 15, 20 ); break;
	case kNPglutSolidTorus : glutSolidTorus( 1.0f, 1.2f, 15, 20 ); break;

	case kNPglutWireDodecahedron : glutWireDodecahedron(); break;
	case kNPglutSolidDodecahedron : glutSolidDodecahedron(); break;
	case kNPglutWireOctahedron : glutWireOctahedron(); break;
	case kNPglutSolidOctahedron : glutSolidOctahedron(); break;
	case kNPglutWireTetrahedron : glutWireTetrahedron(); break;
	case kNPglutSolidTetrahedron : glutSolidTetrahedron(); break;
	case kNPglutWireIcosahedron : glutWireIcosahedron(); break;
	case kNPglutSolidIcosahedron : glutSolidIcosahedron(); break;

	case kNPglutWireTeapot : glutWireTeapot( 2.0f ); break;
	case kNPglutSolidTeapot : glutSolidTeapot( 2.0f ); break;

	default : glutWireTetrahedron(); break;
	}
}